@@ -6,6 +6,7 @@ from enum import StrEnum
 import hashlib
 from http import HTTPMethod
 import logging
+import statistics
 from typing import Any
 
 import aiohttp
@@ -18,6 +19,7 @@ from tenacity import (
     wait_exponential,
 )
 
+from pypetkitapi import utils
 from pypetkitapi.bluetooth import BluetoothManager
 from pypetkitapi.command import ACTIONS_MAP
 from pypetkitapi.const import (
@@ -35,8 +37,10 @@ from pypetkitapi.const import (
     FEEDER_WITH_CAMERA,
     LITTER_NO_CAMERA,
     LITTER_WITH_CAMERA,
+    LIVE_DATA,
     LOGIN_DATA,
     PET,
+    PTK_DBG,
     RES_KEY,
     T3,
     T4,
@@ -49,6 +53,7 @@ from pypetkitapi.const import (
 from pypetkitapi.containers import (
     AccountData,
     Device,
+    LiveFeed,
     Pet,
     PetDetails,
     RegionInfo,
@@ -98,6 +103,7 @@ class PetKitClient:
         region: str,
         timezone: str,
         session: aiohttp.ClientSession | None = None,
+        **kwargs,
     ) -> None:
         """Initialize the PetKit Client."""
 
@@ -115,20 +121,30 @@ class PetKitClient:
         self.petkit_entities: dict[
             int, Feeder | Litter | WaterFountain | Purifier | Pet
         ] = {}
-        self.aiohttp_session = session or aiohttp.ClientSession()
         self.req = PrepReq(
             base_url=PetkitDomain.PASSPORT_PETKIT,
-            session=self.aiohttp_session,
+            session=session,
             timezone=self.timezone,
+            **kwargs,
         )
-        self.bluetooth_manager = BluetoothManager(self)
+        self.bluetooth_manager = BluetoothManager(self, **kwargs)
+        self._debug_test = kwargs.get(PTK_DBG, False)
         from pypetkitapi import MediaManager
 
         from . import __version__
 
-        self.media_manager = MediaManager()
+        self.media_manager = MediaManager(**kwargs)
 
         _LOGGER.debug("PetKit Client initialized (version %s)", __version__)
+        if self._debug_test:
+            _LOGGER.info(
+                "WARNING: pypetkitapi library is in DEBUG_TEST mode. Disable it if you are not developing."
+            )
+
+            _LOGGER.info("Installed packages :")
+            packages = utils.get_installed_packages()
+            for pkg in packages:
+                _LOGGER.info(pkg)
 
     async def _get_base_url(self) -> None:
         """Get the list of API servers, filter by region, and return the matching server."""
@@ -181,7 +197,7 @@ class PetKitClient:
         # Prepare the data to send
         client_nfo = CLIENT_NFO.copy()
         client_nfo["timezoneId"] = self.timezone
-        client_nfo["timezone"] = get_timezone_offset(self.timezone)
+        client_nfo["timezone"] = await get_timezone_offset(self.timezone)
 
         data = LOGIN_DATA.copy()
         data["client"] = str(client_nfo)
@@ -194,9 +210,7 @@ class PetKitClient:
             data["validCode"] = valid_code
         else:
             _LOGGER.debug("Login method: using password")
-            data["password"] = hashlib.md5(
-                self.password.encode()
-            ).hexdigest()  # noqa: S324
+            data["password"] = hashlib.md5(self.password.encode()).hexdigest()
 
         # Send the login request
         response = await self.req.request(
@@ -301,11 +315,14 @@ class PetKitClient:
             await self._get_account_data()
 
         device_list = self._collect_devices()
-        main_tasks, record_tasks, media_tasks = self._prepare_tasks(device_list)
+        main_tasks, record_tasks, media_tasks, live_tasks = self._prepare_tasks(
+            device_list
+        )
 
         await asyncio.gather(*main_tasks)
         await asyncio.gather(*record_tasks)
         await asyncio.gather(*media_tasks)
+        await asyncio.gather(*live_tasks)
         await self._execute_stats_tasks()
 
         end_time = datetime.now()
@@ -324,13 +341,16 @@ class PetKitClient:
                 _LOGGER.debug("Found %s devices", len(account.device_list))
         return device_list
 
-    def _prepare_tasks(self, device_list: list[Device]) -> tuple[list, list, list]:
+    def _prepare_tasks(
+        self, device_list: list[Device]
+    ) -> tuple[list, list, list, list]:
         """Prepare main and record tasks based on device types.
         :param device_list: List of devices.
         :return: Tuple of main tasks, record tasks and media tasks.
         """
         main_tasks: list = []
         record_tasks: list = []
+        live_tasks: list = []
         media_tasks: list = []
 
         for device in device_list:
@@ -339,13 +359,15 @@ class PetKitClient:
             if device_type in DEVICES_FEEDER:
                 main_tasks.append(self._fetch_device_data(device, Feeder))
                 record_tasks.append(self._fetch_device_data(device, FeederRecord))
-                self._add_feeder_task_by_type(media_tasks, device_type, device)
+                self._add_feeder_task_by_type(
+                    media_tasks, live_tasks, device_type, device
+                )
 
             elif device_type in DEVICES_LITTER_BOX:
                 main_tasks.append(self._fetch_device_data(device, Litter))
                 record_tasks.append(self._fetch_device_data(device, LitterRecord))
                 self._add_lb_task_by_type(
-                    record_tasks, media_tasks, device_type, device
+                    record_tasks, media_tasks, live_tasks, device_type, device
                 )
 
             elif device_type in DEVICES_WATER_FOUNTAIN:
@@ -357,10 +379,15 @@ class PetKitClient:
             elif device_type in DEVICES_PURIFIER:
                 main_tasks.append(self._fetch_device_data(device, Purifier))
 
-        return main_tasks, record_tasks, media_tasks
+        return main_tasks, record_tasks, media_tasks, live_tasks
 
     def _add_lb_task_by_type(
-        self, record_tasks: list, media_tasks: list, device_type: str, device: Device
+        self,
+        record_tasks: list,
+        live_tasks: list,
+        media_tasks: list,
+        device_type: str,
+        device: Device,
     ) -> None:
         """Add specific tasks for litter box devices.
         :param record_tasks: List of record tasks.
@@ -373,9 +400,10 @@ class PetKitClient:
         if device_type in LITTER_WITH_CAMERA:
             record_tasks.append(self._fetch_device_data(device, PetOutGraph))
             media_tasks.append(self._fetch_media(device))
+            live_tasks.append(self._fetch_device_data(device, LiveFeed))
 
     def _add_feeder_task_by_type(
-        self, media_tasks: list, device_type: str, device: Device
+        self, media_tasks: list, live_tasks: list, device_type: str, device: Device
     ) -> None:
         """Add specific tasks for feeder box devices.
         :param media_tasks: List of media tasks.
@@ -384,6 +412,7 @@ class PetKitClient:
         """
         if device_type in FEEDER_WITH_CAMERA:
             media_tasks.append(self._fetch_media(device))
+            live_tasks.append(self._fetch_device_data(device, LiveFeed))
 
     async def _execute_stats_tasks(self) -> None:
         """Execute tasks to populate pet stats."""
@@ -418,6 +447,7 @@ class PetKitClient:
             | WaterFountainRecord
             | PetOutGraph
             | LitterStats
+            | LiveFeed
         ],
     ) -> None:
         """Fetch the device data from the PetKit servers.
@@ -511,6 +541,19 @@ class PetKitClient:
                 type(entity),
             )
 
+    @data_handler(LIVE_DATA)
+    async def _handle_live_data(self, device: Device, device_data, device_type: str):
+        """Handle device records."""
+        entity = self.petkit_entities.get(device.device_id)
+        if entity and isinstance(entity, (Feeder, Litter)):
+            entity.live_feed = device_data
+            _LOGGER.debug("Device live feed data fetched OK for %s", device_type)
+        else:
+            _LOGGER.warning(
+                "Cannot assign live_data to entity of type %s",
+                type(entity),
+            )
+
     async def get_pets_list(self) -> list[Pet]:
         """Extract and return the list of pets.
         :return: List of pets.
@@ -541,82 +584,207 @@ class PetKitClient:
             return 0
         return end - start
 
-    async def populate_pet_stats(self, stats_data: Litter) -> None:
+    async def populate_pet_stats(self, litter_data: Litter) -> None:
         """Collect data from litter data to populate pet stats.
-        :param stats_data: Litter data.
+        :param litter_data: Litter data.
         """
-        if not stats_data.device_nfo:
+        if not litter_data.device_nfo:
             _LOGGER.warning(
-                "No device info for %s can't populate pet infos", stats_data
+                "No device info for %s can't populate pet infos", litter_data
             )
             return
 
         pets_list = await self.get_pets_list()
         for pet in pets_list:
-            if stats_data.device_nfo.device_type in [T3, T4]:
-                await self.init_pet_stats(pet)
-                await self._process_litter_no_camera(pet, stats_data)
-            elif stats_data.device_nfo.device_type in [T5, T6]:
-                await self.init_pet_stats(pet)
-                await self._process_litter_camera(pet, stats_data)
+            if litter_data.device_nfo.device_type in [T3, T4]:
+                await self.init_pet_stats(pet, litter_data)
+                await self._process_litter_no_camera(pet, litter_data)
+            elif litter_data.device_nfo.device_type in [T5, T6]:
+                await self.init_pet_stats(pet, litter_data)
+                await self._process_litter_camera(pet, litter_data)
 
     @staticmethod
-    async def init_pet_stats(pet: Pet) -> None:
+    async def init_pet_stats(pet: Pet, litter_data: Litter) -> None:
         """Initialize pet stats.
         Allow pet stats to be displayed in HA even if no data is available.
         :param pet: Pet data.
+        :param litter_data: Litter data.
         """
-        pet.last_litter_usage = 0
-        pet.last_device_used = "Unknown"
-        pet.last_duration_usage = 0
-        pet.last_measured_weight = 0
+        if (
+            getattr(pet, "last_litter_usage", None) is None
+            and getattr(pet, "last_device_used", None) is None
+            and getattr(pet, "last_duration_usage", None) is None
+            and getattr(pet, "last_measured_weight", None) is None
+        ):
+            pet.last_litter_usage = 0
+            pet.last_device_used = "Unknown"
+            pet.last_duration_usage = 0
+            pet.last_measured_weight = 0
 
-    async def _process_litter_no_camera(self, pet: Pet, device_records: Litter) -> None:
+        # Initialize yowling_detected if voice == 1
+        if (
+            getattr(pet, "yowling_detected", None) is None
+            and litter_data.settings
+            and getattr(litter_data.settings, "voice", None) == 1
+        ):
+            pet.yowling_detected = 0
+
+        # Initialize PH-related fields if ph_detection == 1
+        if (
+            getattr(pet, "abnormal_ph_detected", None) is None
+            and getattr(pet, "measured_ph", None) is None
+            and getattr(pet, "soft_stool_detected", None) is None
+            and getattr(pet, "last_urination", None) is None
+            and getattr(pet, "last_defecation", None) is None
+            and litter_data.settings
+            and getattr(litter_data.settings, "ph_detection", None) == 1
+        ):
+            pet.abnormal_ph_detected = 0
+            pet.measured_ph = 7
+            pet.soft_stool_detected = 0
+            pet.last_urination = 0
+            pet.last_defecation = 0
+
+    @staticmethod
+    def set_if_not_none(obj, attr, value):
+        """Set the attribute of an object if the value is not None.
+        :param obj: Object to set the attribute on.
+        :param attr: Attribute to set.
+        :param value: Value to set.
+        """
+        if value is not None:
+            setattr(obj, attr, value)
+
+    async def _process_litter_no_camera(self, pet: Pet, litter_data: Litter) -> None:
         """Process litter T3/T4 records (litter without camera).
         :param pet: Pet data.
-        :param device_records: Litter data.
+        :param litter_data: Litter data.
         """
         for stat in (
-            s
-            for s in device_records.device_records or []
-            if isinstance(s, LitterRecord)
+            s for s in litter_data.device_records or [] if isinstance(s, LitterRecord)
         ):
             if stat.pet_id == pet.pet_id and (
                 pet.last_litter_usage is None
-                or self.get_safe_value(stat.timestamp) > pet.last_litter_usage
+                or getattr(stat, "timestamp", 0) > pet.last_litter_usage
             ):
-                pet.last_litter_usage = (
-                    stat.timestamp if stat.timestamp is not None else 0
+                self.set_if_not_none(pet, "last_litter_usage", stat.timestamp)
+                self.set_if_not_none(
+                    pet,
+                    "last_measured_weight",
+                    getattr(stat.content, "pet_weight", None) if stat.content else None,
                 )
-                pet.last_measured_weight = self.get_safe_value(
-                    stat.content.pet_weight if stat.content else None
+                self.set_if_not_none(
+                    pet,
+                    "last_duration_usage",
+                    (
+                        self.calculate_duration(
+                            stat.content.time_in, stat.content.time_out
+                        )
+                        if stat.content
+                        else None
+                    ),
                 )
-                pet.last_duration_usage = self.calculate_duration(
-                    stat.content.time_in if stat.content else None,
-                    stat.content.time_out if stat.content else None,
+                device_name = getattr(litter_data.device_nfo, "device_name", None)
+                self.set_if_not_none(
+                    pet,
+                    "last_device_used",
+                    device_name.capitalize() if device_name is not None else None,
                 )
-                pet.last_device_used = getattr(
-                    device_records.device_nfo, "device_name", "Unknown"
-                ).capitalize()
 
-    async def _process_litter_camera(self, pet: Pet, pet_graphs: Litter) -> None:
+    async def _process_litter_camera(self, pet: Pet, litter_data: Litter) -> None:
         """Process litter T5/T6 records (litter WITH camera).
         :param pet: Pet data.
-        :param pet_graphs: Litter data.
+        :param litter_data: Litter data.
         """
-        for graph in pet_graphs.device_pet_graph_out or []:
-            if graph.pet_id == pet.pet_id and (
+        device_records = getattr(litter_data, "device_records", None)
+        device_pet_graph = getattr(litter_data, "device_pet_graph_out", None)
+
+        if not isinstance(device_records, list):
+            device_records = []
+        if not isinstance(device_pet_graph, list):
+            device_pet_graph = []
+
+        # Get last_litter_usage, last_measured_weight, last_duration_usage from PetOutGraph
+        for value in device_pet_graph:
+            if value.pet_id == pet.pet_id and (
                 pet.last_litter_usage is None
-                or self.get_safe_value(graph.time) > pet.last_litter_usage
+                or getattr(value, "time", 0) > pet.last_litter_usage
             ):
-                pet.last_litter_usage = graph.time or 0
-                pet.last_measured_weight = self.get_safe_value(
-                    graph.content.pet_weight if graph.content else None
+                self.set_if_not_none(
+                    pet, "last_litter_usage", getattr(value.content, "time", None)
                 )
-                pet.last_duration_usage = self.get_safe_value(graph.toilet_time) or 0
-                pet.last_device_used = getattr(
-                    pet_graphs.device_nfo, "device_name", "Unknown"
-                ).capitalize()
+                self.set_if_not_none(
+                    pet,
+                    "last_measured_weight",
+                    getattr(value.content, "pet_weight", None),
+                )
+                self.set_if_not_none(
+                    pet, "last_duration_usage", getattr(value, "toilet_time", None)
+                )
+                device_name = getattr(litter_data.device_nfo, "device_name", None)
+                self.set_if_not_none(
+                    pet,
+                    "last_device_used",
+                    device_name.capitalize() if device_name else None,
+                )
+                self.set_if_not_none(pet, "last_event_id", value.event_id or None)
+
+        # Get yowling_detected, anormal_ph_detected, measured_ph, soft_stool_detected, last_urination and last_defecation from LitterRecord
+
+        for value in device_records:
+            if value.pet_id == pet.pet_id and value.event_id == pet.last_event_id:
+                # yowling_detected
+                pet.yowling_detected = value.content.pet_voice if value.content else 0
+
+                # anormal_ph_detected : bool
+                if (
+                    value.sub_content
+                    and value.sub_content[0]
+                    and value.sub_content[0].content
+                ):
+                    pet.abnormal_ph_detected = value.sub_content[0].content.ph_state
+
+                # measured_ph : float | None
+                if (
+                    value.sub_content
+                    and value.sub_content[0]
+                    and value.sub_content[0].content
+                    and value.sub_content[0].content.detection_info
+                ):
+                    pet.measured_ph = statistics.mean(
+                        item["ph"]
+                        for item in value.sub_content[0].content.detection_info
+                    )
+                else:
+                    pet.measured_ph = None
+
+                # soft_stool_detected : bool
+                if (
+                    value.sub_content
+                    and value.sub_content[0]
+                    and value.sub_content[0].content
+                ):
+                    pet.soft_stool_detected = value.sub_content[0].content.soft_stools
+
+                # last_urination : str | None
+                if (
+                    value.sub_content
+                    and value.sub_content[0]
+                    and value.sub_content[0].content
+                    and getattr(value.sub_content[0].content, "urine_bolus", None) == 1
+                    and value.timestamp is not None
+                ):
+                    pet.last_urination = value.timestamp
+
+                # last_defecation : str | None
+                if (
+                    value.sub_content
+                    and value.sub_content[0]
+                    and value.sub_content[0].content
+                    and getattr(value.sub_content[0].content, "hard_stools", None) == 1
+                    and value.timestamp is not None
+                ):
+                    pet.last_defecation = value.timestamp
 
     async def get_cloud_video(self, video_url: str) -> dict[str, str | int] | None:
         """Get the video m3u8 link from the cloud.
@@ -741,25 +909,21 @@ class PetKitClient:
         _LOGGER.debug("Command execution success, API response : %s", res)
         return True
 
-    async def close(self) -> None:
-        """Close the aiohttp session if it was created by the client."""
-        if self.aiohttp_session:
-            await self.aiohttp_session.close()
-
 
 class PrepReq:
     """Prepare the request to the PetKit API."""
 
     def __init__(
-        self, base_url: str, session: aiohttp.ClientSession, timezone: str
+        self, base_url: str, session: aiohttp.ClientSession, timezone: str, **kwargs
     ) -> None:
         """Initialize the request."""
         self.base_url = base_url
         self.session = session
         self.timezone = timezone
-        self.base_headers = self._generate_header()
+        self.base_headers: dict[str, str] = {}
+        self._debug_test = kwargs.pop(PTK_DBG, False)
 
-    def _generate_header(self) -> dict[str, str]:
+    async def _generate_header(self) -> dict[str, str]:
         """Create header for interaction with API endpoint."""
         return {
             "Accept": Header.ACCEPT.value,
@@ -773,13 +937,20 @@ class PrepReq:
             "X-Hour": Header.HOUR.value,
             "X-TimezoneId": self.timezone,
             "X-Api-Version": Header.API_VERSION.value,
-            "X-Timezone": get_timezone_offset(self.timezone),
+            "X-Timezone": await get_timezone_offset(self.timezone),
         }
 
     @retry(
         stop=stop_after_attempt(5),
         wait=wait_exponential(multiplier=1, min=1, max=16),
-        retry=retry_if_exception_type(PetkitServerBusyError),
+        retry=(
+            retry_if_exception_type(PetkitServerBusyError)
+            | retry_if_exception_type(aiohttp.ClientConnectorError)
+            | retry_if_exception_type(aiohttp.ClientOSError)
+            | retry_if_exception_type(aiohttp.ServerDisconnectedError)
+            | retry_if_exception_type(aiohttp.ClientResponseError)
+            | retry_if_exception_type(asyncio.TimeoutError)
+        ),
         reraise=True,
     )
     async def request(
@@ -800,6 +971,9 @@ class PrepReq:
         :param headers: Headers to send.
         :return: Response from the API.
         """
+        if not self.base_headers:
+            self.base_headers = await self._generate_header()
+
         _url = url if full_url else "/".join(s.strip("/") for s in [self.base_url, url])
         _headers = {**self.base_headers, **(headers or {})}
         _LOGGER.debug("Request: %s %s", method, _url)
@@ -813,7 +987,17 @@ class PrepReq:
             ) as resp:
                 return await self._handle_response(resp, _url)
         except aiohttp.ClientConnectorError as e:
+            _LOGGER.warning("Connection error while reaching %s: %s", _url, e)
             raise PetkitTimeoutError(f"Cannot connect to host: {e}") from e
+        except aiohttp.ClientOSError as e:
+            _LOGGER.warning("OS-level client error on %s: %s", _url, e)
+            raise PetkitTimeoutError(f"Client OS error: {e}") from e
+        except aiohttp.ServerDisconnectedError as e:
+            _LOGGER.warning("Server disconnected unexpectedly from %s: %s", _url, e)
+            raise PetkitTimeoutError(f"Server disconnected: {e}") from e
+        except asyncio.TimeoutError as e:
+            _LOGGER.warning("Timeout error while waiting for %s", _url)
+            raise PetkitTimeoutError(f"Request to {_url} timed out") from e
 
     @staticmethod
     async def _handle_response(response: aiohttp.ClientResponse, url: str) -> dict:
