@@ -23,6 +23,7 @@ from pypetkitapi import Feeder, Litter, PetKitClient, RecordsItems, RecordType
 from pypetkitapi.const import (
     FEEDER_WITH_CAMERA,
     LITTER_WITH_CAMERA,
+    PTK_DBG,
     MediaType,
     PetkitEndpoint,
     RecordTypeLST,
@@ -66,7 +67,29 @@ class MediaFile:
 class MediaManager:
     """Class to manage media files from PetKit devices."""
 
-    media_table: list[MediaFile] = []
+    def __init__(self, **kwargs):
+        """Media Manager init"""
+        self.media_table: list[MediaFile] = []
+        self._media_index: dict[tuple[str, MediaType], MediaFile] = {}
+        self._debug_test = kwargs.get(PTK_DBG, False)
+
+    def _add_media_to_table(self, media_file: MediaFile) -> None:
+        """Add file to index"""
+        self.media_table.append(media_file)
+        index_key = (media_file.event_id, media_file.media_type)
+        self._media_index[index_key] = media_file
+
+    def clear_media_table(self) -> None:
+        """Empty index table"""
+        self.media_table.clear()
+        self._media_index.clear()
+
+    def _rebuild_index(self) -> None:
+        """Rebuild index."""
+        self._media_index.clear()
+        for media_file in self.media_table:
+            index_key = (media_file.event_id, media_file.media_type)
+            self._media_index[index_key] = media_file
 
     async def gather_all_media_from_cloud(
         self, devices: list[Feeder | Litter]
@@ -108,7 +131,7 @@ class MediaManager:
         self, storage_path: Path, device_id: int
     ) -> list[MediaFile]:
         """Construct the media file table for disk storage."""
-        self.media_table.clear()
+        self.clear_media_table()
 
         today_str = datetime.now().strftime("%Y%m%d")
         base_path = storage_path / str(device_id) / today_str
@@ -139,17 +162,17 @@ class MediaManager:
     ) -> None:
         """Process a subdirectory to collect media files."""
         if not await aiofiles.os.path.exists(subdir):
-            _LOGGER.debug("Path does not exist, skip: %s", subdir)
             return
 
-        _LOGGER.debug("Scanning files into: %s", subdir)
         entries = await aiofiles.os.scandir(subdir)
+
         for entry in entries:
             media_file = await self._create_media_file(
                 entry, device_id, record_type, subdir, valid_pattern
             )
+
             if media_file:
-                self.media_table.append(media_file)
+                self._add_media_to_table(media_file)
 
     @staticmethod
     async def _create_media_file(
@@ -192,12 +215,10 @@ class MediaManager:
         if not dl_type or not event_type:
             _LOGGER.debug("Missing dl_type or event_type, no downloads")
             return []
-
-        existing_event_ids = {media_file.event_id for media_file in self.media_table}
         return [
             mc
             for mc in media_cloud_list
-            if self._should_process_media(mc, event_type, dl_type, existing_event_ids)
+            if self._should_process_media(mc, event_type, dl_type)
         ]
 
     def _should_process_media(
@@ -205,13 +226,12 @@ class MediaManager:
         media_cloud: MediaCloud,
         event_filter: list[RecordType],
         dl_types: list[MediaType],
-        existing_ids: set[str],
     ) -> bool:
         """Determine if a media should be processed as missing."""
         if self._should_skip_event_type(media_cloud.event_type, event_filter):
             return False
-
-        return self._is_media_missing(media_cloud, dl_types, existing_ids)
+        _LOGGER.debug("Process media => %s ", media_cloud)
+        return self._is_media_missing(media_cloud, dl_types)
 
     @staticmethod
     def _should_skip_event_type(
@@ -224,11 +244,13 @@ class MediaManager:
         return False
 
     def _is_media_missing(
-        self, media_cloud: MediaCloud, dl_types: list[MediaType], existing_ids: set[str]
+        self,
+        media_cloud: MediaCloud,
+        dl_types: list[MediaType],
     ) -> bool:
         """Check if any media type is missing."""
-        missing_image = self._is_image_missing(media_cloud, dl_types, existing_ids)
-        missing_video = self._is_video_missing(media_cloud, dl_types, existing_ids)
+        missing_image = self._is_image_missing(media_cloud, dl_types)
+        missing_video = self._is_video_missing(media_cloud, dl_types)
 
         if missing_image or missing_video:
             self._log_missing_details(media_cloud, missing_image, missing_video)
@@ -236,37 +258,29 @@ class MediaManager:
         return False
 
     def _is_image_missing(
-        self, media_cloud: MediaCloud, dl_types: list[MediaType], existing_ids: set[str]
+        self, media_cloud: MediaCloud, dl_types: list[MediaType]
     ) -> bool:
-        """Check if image is missing."""
+        """Check if image is missing"""
         return bool(
             media_cloud.image
             and MediaType.IMAGE in dl_types
-            and not self._media_exists(
-                media_cloud.event_id, MediaType.IMAGE, existing_ids
-            )
+            and not self._media_exists(media_cloud.event_id, MediaType.IMAGE)
         )
 
     def _is_video_missing(
-        self, media_cloud: MediaCloud, dl_types: list[MediaType], existing_ids: set[str]
+        self, media_cloud: MediaCloud, dl_types: list[MediaType]
     ) -> bool:
-        """Check if video is missing."""
+        """Check if video is missing"""
         return bool(
             media_cloud.video
             and MediaType.VIDEO in dl_types
-            and not self._media_exists(
-                media_cloud.event_id, MediaType.VIDEO, existing_ids
-            )
+            and not self._media_exists(media_cloud.event_id, MediaType.VIDEO)
         )
 
-    def _media_exists(
-        self, event_id: str, media_type: MediaType, existing_ids: set[str]
-    ) -> bool:
-        """Check if media exists in local storage."""
-        for mf in self.media_table:
-            if event_id in existing_ids and mf.media_type == media_type:
-                return True
-        return False
+    def _media_exists(self, event_id: str, media_type: MediaType) -> bool:
+        """Check if media exists - O(1) lookup."""
+        index_key = (event_id, media_type)
+        return index_key in self._media_index
 
     @staticmethod
     def _log_missing_details(
@@ -440,7 +454,7 @@ class MediaManager:
             filepath = f"{litter_id}/{date_str}/{event_type.name.lower()}"
             media_files.append(
                 MediaCloud(
-                    event_id=record.event_id,
+                    event_id=f"{litter_id}_{record.timestamp}",
                     event_type=event_type,
                     device_id=litter_id,
                     user_id=user_id,
@@ -470,7 +484,7 @@ class MediaManager:
                             waste_filepath = f"{litter_id}/{date_str}/{RecordType.WASTE_CHECK.name.lower()}"
                             media_files.append(
                                 MediaCloud(
-                                    event_id=record.event_id,
+                                    event_id=f"{litter_id}_{record.timestamp}",
                                     event_type=RecordType.WASTE_CHECK,
                                     device_id=litter_id,
                                     user_id=user_id,
@@ -507,8 +521,8 @@ class MediaManager:
             return "unknown"
         return datetime.fromtimestamp(timestamp).strftime("%Y%m%d")
 
-    @staticmethod
     async def construct_video_url(
+        self,
         device_type: str | None,
         event_data: LitterRecord | RecordsItems,
         user_id: int,
@@ -521,7 +535,11 @@ class MediaManager:
         :param cp_sub: Cpsub value
         :return: Constructed video URL
         """
-        if not hasattr(event_data, "media_api") or not user_id or not cp_sub:
+        if (
+            not hasattr(event_data, "media_api")
+            or not user_id
+            or not (self._debug_test or cp_sub)
+        ):
             return None
         params = parse_qs(str(urlparse(event_data.media_api).query))
         param_dict = {k: v[0] for k, v in params.items()}
@@ -578,10 +596,9 @@ class DownloadDecryptMedia:
         self.file_data = file_data
         if not file_type:
             file_type = []
-        filename = f"{self.file_data.device_id}_{self.file_data.timestamp}"
 
         if self.file_data.image and MediaType.IMAGE in file_type:
-            full_filename = f"{filename}.{MediaType.IMAGE}"
+            full_filename = f"{file_data.event_id}.{MediaType.IMAGE}"
             if await self.not_existing_file(full_filename):
                 # Image download
                 _LOGGER.debug("Download image file (event id: %s)", file_data.event_id)
@@ -592,7 +609,7 @@ class DownloadDecryptMedia:
                 )
 
         if self.file_data.video and MediaType.VIDEO in file_type:
-            if await self.not_existing_file(f"{filename}.{MediaType.VIDEO}"):
+            if await self.not_existing_file(f"{file_data.event_id}.{MediaType.VIDEO}"):
                 # Video download
                 _LOGGER.debug("Download video file (event id: %s)", file_data.event_id)
                 await self._get_video_m3u8()
